@@ -39,52 +39,214 @@
 
 ### Expected metadata columns for GRanges in GFF3 format:
 ###   - required: type, ID
-###   - optional: Parent, Name, Dbxref, geneID
+###   - optional: 1) phase, Parent, Name, Dbxref
+###               2) gene_id, transcript_id, exon_id, protein_id (used
+###                  by Ensembl for their gene, transcript, exon, and CDS
+###                  features, note that exon_id and protein_id are ignored
+###                  at the moment)
+###               3) geneID (used by Flybase)
 ### Used in R/makeTxDbFromGFF.R
-GFF3_COLNAMES <- c("type", "phase", "ID", "Parent", "Name", "Dbxref", "geneID")
+GFF3_COLNAMES <- c("type", "phase", "ID", "Parent", "Name", "Dbxref",
+                   "gene_id", "transcript_id", "exon_id", "protein_id",
+                   "geneID")
 
 ### Expected metadata columns for GRanges in GTF format:
 ###   - required: type, gene_id, transcript_id
-###   - optional: exon_id
+###   - optional: phase, exon_id
 ### Used in R/makeTxDbFromGFF.R
 GTF_COLNAMES <- c("type", "phase", "gene_id", "transcript_id", "exon_id")
 
-### Only valid Sequence Ontology terms that are "gene" offsprings
-### should be added to .GENE_TYPES. Note however that we've made an
-### exception for "pseudogene" which is NOT an offspring of "gene"!
-### Not clear though when exceptions like this are OK or not. For example
-### some GFF3 files (e.g. ref_GRCh38.p12_top_level.gff3.gz found at
-### ftp://ftp.ncbi.nlm.nih.gov/genomes/H_sapiens/GFF/) link some of their
-### exons to parents of type "C_gene_segment", which is neither a "gene" or
-### "transcript" offspring. However, it's not clear that we should make an
-### exception for this term.
-.GENE_TYPES <- c("gene", "pseudogene", "transposable_element_gene")
+### Only valid SO terms (Sequence Ontology terms) that are "gene"
+### offsprings via the **is_a** relationship should be added to .GENE_TYPES.
+.GENE_TYPES <- c(
+    "gene",
 
-### Only valid Sequence Ontology terms that are "transcript" offsprings
-### should be added to .TX_TYPES. Note however that we've made an exception
-### for "pseudogenic_transcript" which is NOT an offspring of "transcript"!
-.TX_TYPES <- c("transcript", "pseudogenic_transcript", "primary_transcript",
-               "mRNA", "ncRNA", "rRNA", "snoRNA", "snRNA", "tRNA", "tmRNA",
-               "miRNA", "miRNA_primary_transcript",
-               "RNase_P_RNA", "RNase_MRP_RNA", "SRP_RNA", "misc_RNA",
-               "antisense_RNA", "antisense",
-               "lnc_RNA", "antisense_lncRNA", "transcript_region",
-               "pseudogenic_tRNA", "scRNA", "guide_RNA", "telomerase_RNA",
-               "vault_RNA", "Y_RNA")
+    ## "protein_coding_gene" and its offsprings:
+    "protein_coding_gene",
+        "gene_with_edited_transcript",
+        "gene_with_mRNA_with_frameshift",
+        "gene_with_polyadenylated_mRNA",
+        "gene_with_recoded_mRNA",
+            "gene_with_mRNA_recoded_by_translational_bypass",
+            "gene_with_transcript_with_translational_frameshift",
+            "gene_with_stop_codon_read_through",
+                "gene_with_stop_codon_redefined_as_pyrrolysine",
+                "gene_with_stop_codon_redefined_as_selenocysteine",
 
-### Only valid Sequence Ontology terms that are "exon" offsprings
-### should be added to .EXON_TYPES. Note however that we've made an
-### exception for "pseudogenic_exon" which is NOT an offspring of "exon"!
-.EXON_TYPES <- c("exon", "pseudogenic_exon", "coding_exon",
-                 "five_prime_coding_exon", "interior_coding_exon",
-                 "three_prime_coding_exon", "exon_of_single_exon_gene",
-                 "interior_exon", "noncoding_exon",
-                 "five_prime_noncoding_exon", "three_prime_noncoding_exon")
+    ## "ncRNA_gene" and its offsprings:
+    "ncRNA_gene",
+        "snoRNA_gene",
+        "gRNA_gene",
+        "RNase_P_RNA_gene",
+        "tmRNA_gene",
+        "SRP_RNA_gene",
+        "tRNA_gene",
+        "rRNA_gene",
+            "rRNA_25S_gene",
+            "rRNA_16S_gene",
+            "rRNA_21S_gene",
+            "rRNA_5S_gene",
+            "rRNA_28S_gene",
+            "rRNA_23S_gene",
+            "rRNA_5_8S_gene",
+            "rRNA_18S_gene",
+        "lncRNA_gene",
+            "bidirectional_promoter_lncRNA",
+            "sense_overlap_ncRNA_gene",
+            "lincRNA_gene",
+            "antisense_lncRNA_gene",
+            "sense_intronic_ncRNA_gene",
+        "enzymatic_RNA_gene",
+            "ribozyme_gene",
+        "RNase_MRP_RNA_gene",
+        "snRNA_gene",
+        "telomerase_RNA_gene",
+        "miRNA_gene",
+        "piRNA_gene",
+        "scRNA_gene",
 
-### Only valid Sequence Ontology terms that are "CDS" offsprings
-### should be added to .CDS_TYPES.
-.CDS_TYPES <- c("CDS", "transposable_element_CDS",
-                "CDS_predicted", "edited_CDS")
+    ## "pseudogene" and its offsprings:
+    "pseudogene",
+        "non_processed_pseudogene",
+            "duplicated_pseudogene",
+            "cassette_pseudogene",
+            "nuclear_mt_pseudogene",
+            "translated_unprocessed_pseudogene",
+            "pseudogene_by_unequal_crossing_over",
+            "transcribed_unprocessed_pseudogene",
+            "unitary_pseudogene",
+                "transcribed_unitary_pseudogene",
+                "allelic_pseudogene",
+        "processed_pseudogene",
+            "transcribed_processed_pseudogene",
+            "translated_processed_pseudogene",
+        "polymorphic_pseudogene",
+            "polymorphic_pseudogene_with_retained_intron",
+        "vertebrate_immune_system_pseudogene",
+            "T_cell_receptor_pseudogene",
+                "TR_J_pseudogene",
+                "TR_V_pseudogene",
+            "immunoglobulin_pseudogene",
+                "IG_J_pseudogene",
+                "IG_V_pseudogene",
+                "IG_C_pseudogene",
+        "transposable_element_pseudogene",
+
+    ## "transposable_element_gene" and its only child (no grandchildren):
+    "transposable_element_gene",
+        "engineered_foreign_transposable_element_gene"
+)
+
+### Only valid SO terms (Sequence Ontology terms) that are "transcript"
+### offsprings via the **is_a** relationship should be added to .TX_TYPES.
+### HOWEVER we've made exceptions for:
+###   - "pseudogenic_transcript";
+###   - the 4 children of "vertebrate_immunoglobulin_T_cell_receptor_segment":
+###     "V_gene_segment", "D_gene_segment", "C_gene_segment", and
+###     "J_gene_segment";
+###   - "gene_segment".
+### According to SO these terms are NOT offsprings of "transcript" via
+### the **is_a** relationship. The reason for these exceptions is that Ensembl
+### assigns transcript-like IDs to these features (e.g. ENST00000488147
+### for "pseudogenic_transcript" WASH7P-201, or ENST00000542354 for
+### "V_gene_segment" TRAV1-1-201, or ENSMUST00000167638 for "gene_segment"
+### Gm17472-201), and this is reinforced by their use of
+### "transcript:ENST00000488147", "transcript:ENST00000542354",
+### and "transcript:ENSMUST00000167638", for the corresponding "ID"
+### attribute in the GFF3 files that they produce.
+### In addition, some GFF3 files produced by NCBI (e.g.
+### ref_GRCh38.p12_top_level.gff3.gz found at
+### ftp://ftp.ncbi.nlm.nih.gov/genomes/H_sapiens/GFF/) link
+### some of their exons to parents of type "C_gene_segment".
+.TX_TYPES <- c(
+    "transcript",
+
+    "primary_transcript",
+    "mRNA",
+    "rRNA",
+    "snoRNA",
+    "snRNA",
+    "tRNA",
+    "tmRNA",
+    "miRNA",
+    "miRNA_primary_transcript",
+    "RNase_P_RNA",
+    "RNase_MRP_RNA",
+    "SRP_RNA",
+    "misc_RNA",
+    "antisense_RNA",
+    "antisense",
+    "lnc_RNA",
+    "antisense_lncRNA",
+    "transcript_region",
+    "scRNA",
+    "guide_RNA",
+    "telomerase_RNA",
+    "vault_RNA",
+    "Y_RNA",
+
+    ## "ncRNA" and some of its children:
+    "ncRNA",
+        "antisense_RNA",
+        "three_prime_overlapping_ncrna",
+        "vault_RNA",
+
+    ## "predicted_transcript" and its only child (no grandchildren):
+    "predicted_transcript",
+        "unconfirmed_transcript",
+
+    ## "pseudogenic_transcript" and its offsprings:
+    "pseudogenic_transcript",
+        "pseudogenic_rRNA",
+            "unitary_pseudogenic_rRNA",
+            "allelic_pseudogenic_rRNA",
+            "unprocessed_pseudogenic_rRNA",
+            "processed_pseudogenic_rRNA",
+        "pseudogenic_tRNA",
+            "unitary_pseudogenic_tRNA",
+            "allelic_pseudogenic_tRNA",
+            "processed_pseudogenic_tRNA",
+            "unprocessed_pseudogenic_tRNA",
+
+    ## "vertebrate_immunoglobulin_T_cell_receptor_segment" and its 4
+    ## children (no grandchildren via the **is_a** relationship):
+    "vertebrate_immunoglobulin_T_cell_receptor_segment",
+        "V_gene_segment",
+        "D_gene_segment",
+        "C_gene_segment",
+        "J_gene_segment",
+
+    ## "gene_segment" and its only child (no grandchildren):
+    "gene_segment",
+        "pseudogenic_gene_segment"
+)
+
+### Only valid SO terms (Sequence Ontology terms) that are "exon"
+### offsprings via the **is_a** relationship should be added to .EXON_TYPES.
+.EXON_TYPES <- c(
+    "exon",
+
+    "pseudogenic_exon",
+    "coding_exon",
+    "five_prime_coding_exon",
+    "interior_coding_exon",
+    "three_prime_coding_exon",
+    "exon_of_single_exon_gene",
+    "interior_exon",
+    "noncoding_exon",
+    "five_prime_noncoding_exon",
+    "three_prime_noncoding_exon"
+)
+
+### Only valid SO terms (Sequence Ontology terms) that are "CDS"
+### offsprings via the **is_a** relationship should be added to .CDS_TYPES.
+.CDS_TYPES <- c(
+    "CDS",
+
+    "transposable_element_CDS",
+    "CDS_predicted",
+    "edited_CDS"
+)
 
 .STOP_CODON_TYPES <- "stop_codon"
 
@@ -96,7 +258,7 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
 {
     type <- gr_mcols$type
     if (is.null(type))
-        stop("'gr' must have a \"type\" metadata column")
+        stop(wmsg("'gr' must have a \"type\" metadata column"))
     ## Return a factor where all levels are in use.
     if (is.factor(type)) {
         levels_in_use <- levels(type)[tabulate(type, nbins=nlevels(type)) != 0L]
@@ -135,8 +297,8 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
 .get_transcript_id <- function(gr_mcols, gene_id, type)
 {
     transcript_id <- gr_mcols$transcript_id
-    ## We've seen silly GTF files that contain only lines of type transcript
-    ## but no transcript_id tag.
+    ## We've seen silly GTF files that contain only lines of type "transcript"
+    ## but no "transcript_id" attribute.
     if (!.no_id(gene_id) && .no_id(transcript_id) && all(type %in% .TX_TYPES))
         return(seq_along(type))  # inferred ids
     if (is.null(transcript_id))
@@ -205,14 +367,18 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
     Parent
 }
 
-### Remove "<type_level>:" prefix (like in "gene:Si016158m.g" and
-### "CDS:Si016158m") if present in *all* non-NA elements of 'ID'.
-.infer_Name_from_ID <- function(ID, type_level)
+### Remove "<type_level>:" prefix (like in "gene:Si016158m.g" or
+### "CDS:Si016158m" or "CDS:ENSP00000493376") but only if it's present
+### in *all* non-NA elements of 'ID'.
+.drop_ID_prefix <- function(ID, type_level)
 {
     prefix <- paste0(type_level, ":")
-    if (!all(substr(ID, start=1L, stop=nchar(prefix)) == prefix, na.rm=TRUE))
-        return(ID)
-    substr(ID, start=nchar(prefix)+1L, stop=nchar(ID))
+    if (all(substr(ID, start=1L, stop=nchar(prefix)) == prefix, na.rm=TRUE))
+        return(substr(ID, start=nchar(prefix)+1L, stop=nchar(ID)))
+    prefix <- paste0(type_level, "-")
+    if (all(substr(ID, start=1L, stop=nchar(prefix)) == prefix, na.rm=TRUE))
+        return(substr(ID, start=nchar(prefix)+1L, stop=nchar(ID)))
+    ID
 }
 
 .get_Name <- function(gr_mcols, type, ID, transcript_id)
@@ -232,7 +398,7 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
             next
         idx <- which(type == type_level)
         if (all(is.na(Name[idx])))
-            Name[idx] <- .infer_Name_from_ID(ID[idx], type_level)
+            Name[idx] <- .drop_ID_prefix(ID[idx], type_level)
     }
     Name
 }
@@ -281,8 +447,8 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
     Dbxref
 }
 
-### The FlyBase people use the geneID tag to assign an external gene id to
-### each of their transcripts in their GFF3 files. There is no corresponding
+### The FlyBase people use the "geneID" attribute to assign an external gene
+### id to each transcript in their GFF3 files. There is no corresponding
 ### "gene" line in the file.
 .get_geneID <- function(gr_mcols)
 {
@@ -367,7 +533,10 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
 {
     if (gtf.format)
         return(integer(0))
-    exon_IDX[as.logical(unique(Parent[exon_IDX] %in% ID[gene_IDX]))]
+    has_gene_parent <- as.logical(unique(Parent[exon_IDX] %in% ID[gene_IDX]))
+    ## We've seen silly GFF files where exons have no parents. This introduces
+    ## NAs in 'has_gene_parent' so we use which() here to get rid of them.
+    exon_IDX[which(has_gene_parent)]
 }
 
 ### Returns the index of genes that have exons as direct children. These genes
@@ -418,7 +587,7 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
 {
     tx_ids <- sort(as.character(tx_ids))
     in1string <- paste0(tx_ids, collapse=", ")
-    warning(wmsg("The following transcripts were rejected because ",
+    warning(wmsg("The following transcripts were dropped because ",
                  because, ": ", in1string))
 
     objnames <- .ls_rejected_tx_envir()
@@ -625,51 +794,68 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
     tx_id <- transcripts$tx_id
     if (!is.character(tx_id))
         tx_id <- as.character(tx_id)
+    ## 'transcripts_by_id' is a SplitDataFrameList object.
     transcripts_by_id <- splitAsList(transcripts, tx_id, drop=TRUE)
 
-    tx_id <- names(transcripts_by_id)
-
-    tx_name <- unique(transcripts_by_id[ , "tx_name"])
-    bad_tx <- names(which(elementNROWS(tx_name) != 1L))
-    if (length(bad_tx) != 0L) {
-        in1string <- paste0(sort(bad_tx), collapse=", ")
-        stop(wmsg("The following transcripts have multiple parts that cannot ",
-                  "be merged because of incompatible Name: ", in1string))
+    tx_name <- unique(transcripts_by_id[ , "tx_name"])  # CharacterList
+    drop_idx <- which(elementNROWS(tx_name) != 1L)
+    if (length(drop_idx) != 0L) {
+        in1string <- paste0(sort(names(drop_idx)), collapse=", ")
+        warning(wmsg("The following transcripts were dropped because they ",
+                     "have multiple parts that could not be merged due to ",
+                     "incompatible \"Name\" attributes: ", in1string))
+        transcripts_by_id <- transcripts_by_id[-drop_idx]
+        tx_name <- tx_name[-drop_idx]
     }
     tx_name <- as.character(tx_name)
 
-    tx_type <- unique(transcripts_by_id[ , "tx_type"])
-    bad_tx <- names(which(elementNROWS(tx_type) != 1L))
-    if (length(bad_tx) != 0L) {
-        in1string <- paste0(sort(bad_tx), collapse=", ")
-        stop(wmsg("The following transcripts have multiple parts that cannot ",
-                  "be merged because of incompatible type: ", in1string))
+    tx_type <- unique(transcripts_by_id[ , "tx_type"])  # CharacterList
+    drop_idx <- which(elementNROWS(tx_type) != 1L)
+    if (length(drop_idx) != 0L) {
+        in1string <- paste0(sort(names(drop_idx)), collapse=", ")
+        warning(wmsg("The following transcripts were dropped because they ",
+                     "have multiple parts that could not be merged due to ",
+                     "incompatible types: ", in1string))
+        transcripts_by_id <- transcripts_by_id[-drop_idx]
+        tx_name <- tx_name[-drop_idx]
+        tx_type <- tx_type[-drop_idx]
     }
     tx_type <- as.character(tx_type)
 
-    tx_chrom <- unique(transcripts_by_id[ , "tx_chrom"])
-    bad_tx <- names(which(elementNROWS(tx_chrom) != 1L))
-    if (length(bad_tx) != 0L) {
-        in1string <- paste0(sort(bad_tx), collapse=", ")
-        stop(wmsg("The following transcripts have multiple parts that cannot ",
-                  "be merged because of incompatible seqnames: ", in1string))
+    tx_chrom <- unique(transcripts_by_id[ , "tx_chrom"])  # CharacterList
+    drop_idx <- which(elementNROWS(tx_chrom) != 1L)
+    if (length(drop_idx) != 0L) {
+        in1string <- paste0(sort(names(drop_idx)), collapse=", ")
+        warning(wmsg("The following transcripts were dropped because they ",
+                     "have multiple parts that could not be merged due to ",
+                     "incompatible seqnames: ", in1string))
+        transcripts_by_id <- transcripts_by_id[-drop_idx]
+        tx_name <- tx_name[-drop_idx]
+        tx_type <- tx_type[-drop_idx]
+        tx_chrom <- tx_chrom[-drop_idx]
     }
     tx_chrom <- as.character(tx_chrom)
 
-    tx_strand <- unique(transcripts_by_id[ , "tx_strand"])
-    bad_tx <- names(which(elementNROWS(tx_strand) != 1L))
-    if (length(bad_tx) != 0L) {
-        in1string <- paste0(sort(bad_tx), collapse=", ")
-        stop(wmsg("The following transcripts have multiple parts that cannot ",
-                  "be merged because of incompatible strand: ", in1string))
+    tx_strand <- unique(transcripts_by_id[ , "tx_strand"])  # FactorList
+    drop_idx <- which(elementNROWS(tx_strand) != 1L)
+    if (length(drop_idx) != 0L) {
+        in1string <- paste0(sort(names(drop_idx)), collapse=", ")
+        warning(wmsg("The following transcripts were dropped because they ",
+                     "have multiple parts that could not be merged due to ",
+                     "incompatible strands: ", in1string))
+        transcripts_by_id <- transcripts_by_id[-drop_idx]
+        tx_name <- tx_name[-drop_idx]
+        tx_type <- tx_type[-drop_idx]
+        tx_chrom <- tx_chrom[-drop_idx]
+        tx_strand <- tx_strand[-drop_idx]
     }
     tx_strand <- as.character(tx_strand)
 
     tx_start <- unname(min(transcripts_by_id[ , "tx_start"]))
     tx_end <- unname(max(transcripts_by_id[ , "tx_end"]))
-    
+
     data.frame(
-        tx_id=tx_id,
+        tx_id=names(transcripts_by_id),
         tx_name=tx_name,
         tx_type=tx_type,
         tx_chrom=tx_chrom,
@@ -680,12 +866,28 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
     )
 }
 
-.extract_transcripts_from_GRanges <- function(tx_IDX, gr, type, ID, Name)
+.extract_transcripts_from_GRanges <- function(tx_IDX, gr, type, ID,
+                                              transcript_id, Name)
 {
     tx_id <- ID[tx_IDX]
+    if (is.character(transcript_id) && !all(is.na(transcript_id))) {
+        what <- "transcript_id"
+        tx_name <- transcript_id[tx_IDX]
+    } else {
+        what <- "Name"
+        tx_name <- Name[tx_IDX]
+    }
+    if (anyNA(tx_name))
+        warning(wmsg("some transcripts have no \"", what, "\" attribute ==> ",
+                     "their name (\"tx_name\" column in the TxDb object) ",
+                     "was set to NA"))
+    if (anyDuplicated(tx_name))
+        warning(wmsg("the transcript names (\"tx_name\" column in the TxDb ",
+                     "object) imported from the \"", what, "\" attribute ",
+                     "are not unique"))
     transcripts <- data.frame(
         tx_id=tx_id,
-        tx_name=Name[tx_IDX],
+        tx_name=tx_name,
         tx_type=type[tx_IDX],
         tx_chrom=seqnames(gr)[tx_IDX],
         tx_strand=strand(gr)[tx_IDX],
@@ -696,9 +898,9 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
     merged_tx <- unique(tx_id[duplicated(tx_id)])
     if (length(merged_tx) != 0L) {
         transcripts <- .merge_transcript_parts(transcripts)
-        in1string <- paste0(sort(merged_tx), collapse=", ")
-        warning(wmsg("The following transcripts have multiple parts that ",
-                     "were merged: ", in1string))
+        #in1string <- paste0(sort(merged_tx), collapse=", ")
+        #warning(wmsg("The following transcripts have multiple parts that ",
+        #             "were merged: ", in1string))
     }
     ## We've seen silly GFF/GTF files where each transcript in the file is
     ## reported to be on its own contig and spans it (start=1) but no strand
@@ -737,7 +939,7 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
 
     tx_start <- unname(min(exons_by_id[ , "exon_start"]))
     tx_end <- unname(max(exons_by_id[ , "exon_end"]))
-    
+
     data.frame(
         tx_id=tx_id,
         tx_name=tx_id,
@@ -874,10 +1076,35 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
 ### Extract the 'genes' data frame
 ###
 
+.get_gene_ids <- function(ID, Name, gene_id, gene_IDX)
+{
+    ## First we try to get the gene ids from the "gene_id" attribute.
+    ## For example, gene features in GFF3 files from Ensembl have
+    ## a "gene_id" attribute set to their Ensembl id (e.g. ENSG00000223972)
+    ## so we will import that in the TxDb object instead of the "Name"
+    ## attribute which might contain duplicates.
+    if (!is.null(gene_id)) {
+        gene_ids <- gene_id[gene_IDX]
+        if (!anyNA(gene_ids))
+            return(gene_ids)
+    }
+    ## If we can't use the "gene_id" attribute, then we get the gene ids
+    ## from the "Name" attribute.
+    gene_ids <- Name[gene_IDX]
+    if (!anyNA(gene_ids))
+        return(gene_ids)
+    ## If both attempts failed, we return NULL.
+    NULL
+}
+
 .extract_genes_from_gff3_GRanges <- function(gene_IDX, tx_IDX,
-                                             ID, Name, Parent,
+                                             ID, Parent,
+                                             gene_id, Name,
                                              Dbxref=NULL, geneID=NULL)
 {
+    if (anyNA(ID[gene_IDX]))
+        stop(wmsg("some genes have no \"ID\" attribute"))
+
     tx_id <- ID[tx_IDX]
     tx2genes <- Parent[tx_IDX]
 
@@ -886,9 +1113,9 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
     tx2genes[idx0] <- tx_id[idx0]
 
     ## Transcripts with no parents are sometimes linked to a gene via the
-    ## Dbxref or geneID tag.
+    ## "Dbxref" or "geneID" attribute.
 
-    ## First we try to find their parent via the Dbxref tag, if present.
+    ## First we try to find their parent via the "Dbxref" attribute, if present.
     if (!is.null(Dbxref)) {
         idx0 <- which(elementNROWS(tx2genes) == 0L)
         tx_Dbxref <- Dbxref[tx_IDX[idx0]]
@@ -905,15 +1132,31 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
                                  as(hits, "PartitioningByEnd"))
     }
 
-    ## Replace gene IDs by gene Names and remove NAs.
-    tmp <- unlist(tx2genes, use.names=FALSE)
-    ID2Name <- Name[gene_IDX]
-    names(ID2Name) <- ID[gene_IDX]
-    tx2genes <- relist(unname(ID2Name[tmp]), tx2genes)
+    ## Try to get the gene ids from the "gene_id" or "Name" attributes.
+    gene_ids <- .get_gene_ids(ID, Name, gene_id, gene_IDX)
+    if (is.null(gene_ids)) {
+        gene_ids <- .drop_ID_prefix(ID[gene_IDX], "gene")
+    } else {
+        ## The mapping between 'ID[gene_IDX]' and 'gene_ids' must be
+        ## one-to-one. If it's not, then we can't use 'gene_ids' to reliably
+        ## identify genes so we get the gene ids from the "ID" attribute.
+        sm1 <- match(ID[gene_IDX], ID[gene_IDX])
+        sm2 <- match(gene_ids, gene_ids)
+        if (!identical(sm1, sm2))
+            gene_ids <- .drop_ID_prefix(ID[gene_IDX], "gene")
+    }
+
+    ## Replace the gene IDs in the 'tx2genes' mapping with the gene ids
+    ## in 'gene_ids'.
+    names(gene_ids) <- ID[gene_IDX]
+    gene_ids <- gene_ids[unlist(tx2genes, use.names=FALSE)]
+    tx2genes <- relist(unname(gene_ids), tx2genes)
+
+    ## Remove NAs.
     tx2genes <- tx2genes[!is.na(tx2genes)]
 
-    ## Then, if we still have transcripts with no parent, we use the geneID
-    ## tag (if present) to assign them an external gene id.
+    ## Then, if we still have transcripts with no parent, we use the "geneID"
+    ## attribute (if present) to assign them an external gene id.
     if (!is.null(geneID)) {
         idx0 <- which(elementNROWS(tx2genes) == 0L)
         tx2genes[idx0] <- geneID[tx_IDX[idx0]]
@@ -1084,7 +1327,9 @@ makeTxDbFromGRanges <- function(gr, drop.stop.codons=FALSE, metadata=NULL,
     }
     transcripts <- .extract_transcripts_from_GRanges(tx_IDX, gr,
                                                      mcols0$type,
-                                                     mcols0$ID, mcols0$Name)
+                                                     mcols0$ID,
+                                                     mcols0$transcript_id,
+                                                     mcols0$Name)
     if (gtf.format) {
         transcripts <- .add_missing_transcripts(transcripts, exons)
         genes <- .extract_genes_from_gtf_GRanges(mcols0$transcript_id,
@@ -1094,8 +1339,8 @@ makeTxDbFromGRanges <- function(gr, drop.stop.codons=FALSE, metadata=NULL,
         Dbxref <- .get_Dbxref(gr_mcols)
         geneID <- .get_geneID(gr_mcols)
         genes <- .extract_genes_from_gff3_GRanges(gene_IDX, tx_IDX,
-                                                  mcols0$ID, mcols0$Name,
-                                                  mcols0$Parent,
+                                                  mcols0$ID, mcols0$Parent,
+                                                  mcols0$gene_id, mcols0$Name,
                                                   Dbxref, geneID)
     }
 
@@ -1360,8 +1605,8 @@ gr6 <- import(file6, format="gff3", colnames=GFF3_COLNAMES,
 ## (a) makeTxDbFromGFF() fails on TheCanonicalGene_v1.gff3
 ##
 ## (b) gene_id, tx_name, exon_name, and cds_name are now imported from the
-##     Name tag instead of the ID tag (GFF3 Spec: "IDs do not have meaning
-##     outside the file in which they reside")
+##     "Name" attribute instead of the "ID" attribute (GFF3 Spec: "IDs do
+##     not have meaning outside the file in which they reside")
 
 
 ## Test with GRanges obtained from GTF files
