@@ -205,8 +205,8 @@
 ### reasonable time. For this we need to be able to retrieve all the "central
 ### tables" for all the transcript-centric tracks available for a given
 ### organism. Using a combination of calls to rtracklayer::trackNames(session)
-### and rtracklayer::tableNames(ucscTableQuery(session, track=track)) would
-### partly achieve this but is unfortunately very slow.
+### and rtracklayer::ucscTables(genome(session), track) would partly achieve
+### this but is unfortunately very slow.
 supportedUCSCtables <- function(genome="hg19",
                                 url="http://genome.ucsc.edu/cgi-bin/")
 {
@@ -235,9 +235,8 @@ supportedUCSCtables <- function(genome="hg19",
     ## track has subtracks (e.g. the Augustus track for hg18 has 3 subtracks),
     ## in which case there is 1 "central table" per subtrack. So trackNames()
     ## alone cannot be used to get the one-to-many mapping from tracks to
-    ## "central tables". Calling tableNames(ucscTableQuery(session,
-    ## track=track)) in a loop on all the tracks returned by trackNames()
-    ## would work but is very slow :-/
+    ## "central tables". Calling ucscTables(genome(session), track) in a loop
+    ## on all the tracks returned by trackNames() would work but is very slow.
     genome_tracknames <- trackNames(session)
 
     ## Resolve NAs in the tablename column.
@@ -285,7 +284,10 @@ browseUCSCtrack <- function(genome="hg19",
     if (!isSingleString(url))
         stop("'url' must be a single string")
     url <- sprintf("%s/hgTrackUi?db=%s&g=%s", url, genome, tablename)
-    browseURL(url)
+    ## Avoid "file association for 'http://...' not available or invalid"
+    ## error during 'R CMD check' on some Windows systems (e.g. riesling1).
+    if (interactive())
+        browseURL(url)
 }
 
 .tablename2track <- function(tablename, session)
@@ -299,7 +301,7 @@ browseUCSCtrack <- function(genome="hg19",
     ## Sanity check.
     stopifnot(length(idx) == 1L)  # should never happen
     track <- as.character(supported_tables$track[idx])
-    track_tables <- tableNames(ucscTableQuery(session, track=track))
+    track_tables <- ucscTables(genome(session), track)
     if (!(tablename %in% track_tables))
         stop("UCSC table \"", tablename, "\" does not exist ",
              "for genome \"", genome(session), "\", sorry")
@@ -328,6 +330,14 @@ browseUCSCtrack <- function(genome="hg19",
         ),
         gene_id_type="Entrez Gene ID"
     ),
+    ncbiRefSeq=list(
+        L2Rchain=list(
+            c(tablename="ncbiRefSeqLink",
+              Lcolname="id",
+              Rcolname="locusLinkId")
+        ),
+        gene_id_type="Entrez Gene ID"
+    ),
     vegaGene=list(
         L2Rchain=list(
             c(tablename="vegaGtp",
@@ -551,9 +561,11 @@ browseUCSCtrack <- function(genome="hg19",
     on.exit(message("OK"))
     ucsc_txtable <- UCSC_dbselect(genome, tablename,
                                   columns=columns, where=where)
-    current_classes <- head(sapply(ucsc_txtable, class),
-                            n=length(.UCSC_TXCOL2CLASS))
-    stopifnot(identical(current_classes, .UCSC_TXCOL2CLASS))
+    ## new check because exon starts and ends are blobs now
+    stopifnot(all(mapply(function(x, y) is(x, y), ucsc_txtable, .UCSC_TXCOL2CLASS)))
+    ##current_classes <- head(sapply(ucsc_txtable, class),
+    ##                        n=length(.UCSC_TXCOL2CLASS))
+    ##stopifnot(identical(current_classes, .UCSC_TXCOL2CLASS))
     ucsc_txtable$exonStarts <- toListOfIntegerVectors(ucsc_txtable$exonStarts)
     ucsc_txtable$exonEnds <- toListOfIntegerVectors(ucsc_txtable$exonEnds)
     if (!identical(lengths(ucsc_txtable$exonStarts),
